unit main;

// Code Example of using XDOM 2.4.0
// Delphi implementation
//
// You need XDOM 2.4.0 or above to use this source code.
// The latest version of XDOM can be found at "http://www.philo.de/xml/".
//
// This  example  source code  shows  how  to
// process  and test  an XML file  for  well-
// formedness using the XDOM Delphi unit, and 
// how to copy  one DOM tree to another using
// some  of XDOM's special purpose  processor 
// classes.

interface

uses
  XDOM_2_4,
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons;

type

  TForm1 = class(TForm)
    XmlStandardDocReader1: TXmlStandardDocReader;
    XmlWFTestContentHandler1: TXmlWFTestContentHandler;
    SpeedButton1: TSpeedButton;
    OpenDialog1: TOpenDialog;
    Memo1: TMemo;
    DomImplementation1: TDomImplementation;
    XmlStandardDomReader1: TXmlStandardDomReader;
    BitBtn1: TBitBtn;
    Memo2: TMemo;
    XmlStandardHandler1: TXmlStandardHandler;
    ListBox1: TListBox;
    Label3: TLabel;
    XmlDistributor1: TXmlDistributor;
    Memo3: TMemo;
    BitBtn2: TBitBtn;
    XmlWFTestDTDHandler1: TXmlWFTestDTDHandler;
    XmlStandardCMReader1: TXmlStandardCMReader;
    XmlStandardDtdReader1: TXmlStandardDtdReader;
    DomToXmlParser1: TDomToXmlParser;
    CMToXmlParser1: TCMToXmlParser;
    XmlDocBuilder1: TXmlDocBuilder;
    XmlDocBuilder2: TXmlDocBuilder;
    XmlCMBuilder1: TXmlCMBuilder;
    XmlCMBuilder2: TXmlCMBuilder;
    procedure SpeedButton1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure BitBtn1Click(Sender: TObject);
    procedure BitBtn2Click(Sender: TObject);
    procedure DomImplementation1Error(sender: TObject; error: TdomError;
      var continue: Boolean);
    procedure DomImplementation1ExternalParsedEntity(sender: TObject;
      const parentSystemId: WideString; var publicId, systemId: WideString;
      var stream: TStream; var action: TXmlParserAction);
    procedure XmlStandardHandler1StartDocument(reader: TXmlCustomReader;
      locator: TdomLocator; var version, encDl: WideString;
      var sdDl: TdomStandalone);
    procedure XmlStandardHandler1StartElement(reader: TXmlCustomReader;
      locator: TdomLocator; var tagName: WideString;
      attributes: TdomNameValueList);
  private
    { Private declarations }
    function getXmlCM1: TdomCMObject;
    function getXmlCM2: TdomCMObject;
    function getXmlDoc1: TdomDocument;
    function getXmlDoc2: TdomDocument;
  public
    { Public declarations }
    property XmlCM1: TdomCMObject read getXmlCM1;
    property XmlCM2: TdomCMObject read getXmlCM2;
    property XmlDoc1: TdomDocument read getXmlDoc1;
    property XmlDoc2: TdomDocument read getXmlDoc2;
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}


// TForm1
// ------

function TForm1.getXmlCM1: TdomCMObject;
begin
  Result:= TdomCMObject(DomImplementation1.cmObjects[0]);
end;

function TForm1.getXmlCM2: TdomCMObject;
begin
  Result:= TdomCMObject(DomImplementation1.cmObjects[1]);
end;

function TForm1.getXmlDoc1: TdomDocument;
begin
  Result:= (DomImplementation1.documents.item(0) as TdomDocument);
end;

function TForm1.getXmlDoc2: TdomDocument;
begin
  Result:= (DomImplementation1.documents.item(1) as TdomDocument);
end;

procedure TForm1.SpeedButton1Click(Sender: TObject);
var
  UpTime,ParsingTime: dword;
  ok: boolean;
  XMLStream: TdomWideStringStream;
  MStream: TMemoryStream;
  InputSrc: TXmlInputSource;
  intSubsetData,intSubsetSysId,extSubsetPubId,extSubsetSysId: wideString;
  extDtdStream: TStream;
  action: TXmlParserAction;
  S: string;
begin
  if OpenDialog1.Execute then begin

    if not FileExists (OpenDialog1.FileName) then begin
      Memo1.text:= 'File not found!';
      exit;
    end;

    Memo1.Clear;
    Memo1.Update;

    xmlDoc1.clear;
    xmlCM1.externalCM.clear;
    xmlCM1.internalCM.clear;
    XmlDocBuilder1.referenceNode:= xmlDoc1;
    XmlCMBuilder1.extContentModel:= xmlCM1.externalCM;
    XmlCMBuilder1.intContentModel:= xmlCM1.internalCM;

    ListBox1.Items.BeginUpdate;
    try
      UpTime:= GetTickCount;

      // Parse the XML File
      // ----------------------
      MStream:= TMemoryStream.create; // First, we need a stream for the Input Source later
      try
        MStream.LoadFromFile(OpenDialog1.Filename);  // Here we load the entire contents of the XML file into the stream's memory buffer.
        InputSrc:= TXmlInputSource.create(MStream,'',OpenDialog1.Filename,1,4096); // Now, we create the Input Source
        try
          ok:= XmlStandardDocReader1.parse(InputSrc); // Voila! We parse the Input Source
        finally
          InputSrc.free;
        end; {try}
      finally
        MStream.free;
      end; {try}

      if ok                           // error while parsing?
        and assigned(xmlDoc1.doctypeDecl) // Doctype Declaration existing?
        then begin

        // Parse the DTD
        // -------------
        intSubsetSysId:= xmlDoc1.systemId;
        with xmlDoc1.doctypeDecl do begin
          intSubsetData:= internalSubset;
          extSubsetPubId:= publicId;
          extSubsetSysId:= systemId;
        end;

            // Here we parse the internal subset of the DTD
            // --------------------------------------------
        if intSubsetData <> '' then begin
          if intSubsetData[1] <> #$feff
            then  intSubsetData:= concat(wideString(#$feff),intSubsetData); // add the byte order mark, which is required for the Input Source
          XmlStream:= TdomWideStringStream.create(intSubsetData);
          try
            InputSrc:= TXmlInputSource.create(XmlStream,'',intSubsetSysId,1,4096);
            try
              ok:= XmlStandardDtdReader1.parseInternalSubset(InputSrc,true);
            finally
              InputSrc.free;
            end;
          finally
            XmlStream.free;
          end;
        end;

        if ok then begin

              // Here we parse the external subset of the DTD
              // --------------------------------------------
          if (extSubsetPubId <> '') or (extSubsetSysId <> '') then begin
            extDtdStream:= nil;
            action:= paFail;
            try

              // Lookup the external file
              if assigned(DomImplementation1.OnExternalParsedEntity) then begin
                repeat
                  DomImplementation1.OnExternalParsedEntity(self,intSubsetSysId,extSubsetPubId,extSubsetSysId,extDtdStream,action);
                until action <> paRetry;
              end;

              if (action = paFail) or not assigned(extDtdStream) then begin // Lookup successful?
                ok:= false;
              end else begin
                InputSrc:= TXmlInputSource.create(extDtdStream,extSubsetPubId,extSubsetSysId,1,4096);
                try
                  ok:= XmlStandardDtdReader1.parseExternalSubset(InputSrc);
                finally
                  InputSrc.free;
                end; {try}
              end;

            finally
              if assigned(extDtdStream) then extDtdStream.free;
            end; {try ... finally ...}
          end; {if (extSubsetPubId <> '') ...}

        end; {if ok ...}

      end; {if ok ...}

      ParsingTime:= GetTickCount-UpTime;
    finally
      ListBox1.Items.EndUpdate;
    end;
    if ok then begin
      DomToXmlParser1.writeToString(xmlDoc1,'Latin1',S);
      memo1.Text:= S;
      memo1.Lines.Add('');
      memo1.Lines.Add('');
      memo1.Lines.Add('OK.');
    end else begin
      memo1.Lines.Add('');
      memo1.Lines.Add('');
      memo1.Lines.Add('TERMINATED.');
    end;
    memo1.Lines.Add('ParsingTime: '+IntToStr(ParsingTime)+' msec.');
    memo1.update;

  end; {if OpenDialog1.Execute ...}
end;

procedure TForm1.BitBtn1Click(Sender: TObject);
var
  UpTime,ParsingTime: dword;
  ok: boolean;
  S: string;
begin
    Memo2.Clear;
    Memo2.Update;

    xmlDoc2.clear;
    XmlDocBuilder2.referenceNode:= xmlDoc2;
    UpTime:= GetTickCount;
    ok:= XmlStandardDomReader1.parse(xmlDoc1);
    ParsingTime:= GetTickCount-UpTime;
    if ok then begin
      DomToXmlParser1.writeToString(xmlDoc2,'Latin1',S);
      Memo2.Text:= S;
      Memo2.Lines.Add('');
      Memo2.Lines.Add('');
      Memo2.Lines.Add('OK.');
    end else begin
      Memo2.Lines.Add('');
      Memo2.Lines.Add('');
      Memo2.Lines.Add('TERMINATED.');
    end;
    Memo2.Lines.Add('ParsingTime: '+IntToStr(ParsingTime)+' msec.');
    Memo2.update;
end;

procedure TForm1.FormCreate(Sender: TObject);
var
  doc: TdomDocument;
  n: integer;
begin
  for n:= 1 to 2 do begin
    // Document #n:
    doc:= DomImplementation1.createDocument('dummy',nil);
    doc.clear;  // Delete the dummy root element
    // Content Model #n:
    DomImplementation1.createCMObject;
  end;
end;

procedure TForm1.DomImplementation1ExternalParsedEntity(sender: TObject;
  const parentSystemId: WideString; var publicId, systemId: WideString;
  var stream: TStream; var action: TXmlParserAction);
var
  path: TFileName;
begin
  if systemId <> '' then begin
    try
      // Resolve path:
         path := concat(ExtractFileDir(parentSystemId),'\',SystemID);
      // This works only with relative system identifiers of
      // certain kinds.

      systemId:= path;
      stream:= TFileStream.Create(path, fmOpenRead);
      action:= paOK;
    except
      action:= paFail;
    end;
  end;
end;

procedure TForm1.BitBtn2Click(Sender: TObject);
var
  UpTime,ParsingTime: dword;
  ok: boolean;
  S: string;
begin
    memo3.Clear;
    memo3.Update;

    xmlCM2.externalCM.clear;
    XmlCMBuilder2.extContentModel:= xmlCM2.externalCM;
    UpTime:= GetTickCount;
    ok:= XmlStandardCMReader1.parse(xmlCM1.externalCM);
    ParsingTime:= GetTickCount-UpTime;
    if ok then begin
      CMToXmlParser1.writeToString(xmlCM1.externalCM,'Latin1',S);
      memo3.Text:= S;
      memo3.Lines.Add('');
      memo3.Lines.Add('');
      memo3.Lines.Add('OK.');
    end else begin
      memo3.Lines.Add('');
      memo3.Lines.Add('');
      memo3.Lines.Add('TERMINATED.');
    end;
    memo3.Lines.Add('ParsingTime: '+IntToStr(ParsingTime)+' msec.');
    memo3.update;
end;

procedure TForm1.DomImplementation1Error(sender: TObject; error: TdomError;
  var continue: Boolean);
begin
  error.language:= iso639_en;
  if (sender = XmlStandardDocReader1) or (sender = XmlStandardDtdReader1)
    then Form1.memo1.Lines.Add(error.message)
    else Form1.memo2.Lines.Add(error.message);
end;

procedure TForm1.XmlStandardHandler1StartDocument(reader: TXmlCustomReader;
  locator: TdomLocator; var version, encDl: WideString;
  var sdDl: TdomStandalone);
begin
  ListBox1.clear;
end;

procedure TForm1.XmlStandardHandler1StartElement(reader: TXmlCustomReader;
  locator: TdomLocator; var tagName: WideString;
  attributes: TdomNameValueList);
begin
  with ListBox1.Items do
    if IndexOf(tagName) = -1  // If the tag name is not already present in the ListBox, ...
      then Add(tagName);      // then add the tag name to the ListBox.
end;

end.
