unit main;

// Code Example of using XDOM 2.4.0
// Delphi implementation
//
// You need XDOM 2.4.0 or above to use this source code.
// The latest version of XDOM can be found at "http://www.philo.de/xml/".
//
// This example source code shows how to load
// and  display  the source  of XML files and
// their entities using the XDOM Delphi unit.

interface

uses
  XDOM_2_4, TypInfo,
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, Tabs, FileCtrl, ExtCtrls, ComCtrls;

type
  TMainpage = class(TForm)
    Label3: TLabel;
    Label4: TLabel;
    OpenDialog1: TOpenDialog;
    SpeedButton1: TSpeedButton;
    SpeedButton2: TSpeedButton;
    TabControl1: TTabControl;
    Memo1: TMemo;
    TabSet1: TTabSet;
    Memo2: TMemo;
    TreeView1: TTreeView;
    XmlToDomParser1: TXmlToDomParser;
    DomImplementation1: TDomImplementation;
    GroupBox1: TGroupBox;
    RadioGroup1: TRadioGroup;
    Button1: TButton;
    DomToXmlParser1: TDomToXmlParser;
    Label5: TLabel;
    Label6: TLabel;
    Panel1: TPanel;
    Label2: TLabel;
    Label1: TLabel;
    RadioGroup2: TRadioGroup;
    RadioGroup3: TRadioGroup;
    procedure OpenFile(Sender: TObject);
    procedure TabSet1Click(Sender: TObject);
    procedure CloseFile(Sender: TObject);
    procedure TabControl1Change(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure DomImplementation1ExternalParsedEntity(sender: TObject;
      const parentSystemId: WideString; var publicId, systemId: WideString;
      var stream: TStream; var action: TXmlParserAction);
    procedure FormCreate(Sender: TObject);
    procedure ReportError(sender: TObject; error: TdomError;
      var continue: Boolean);
  private
    { Private-Deklarationen }
    procedure UpdateTreeView(const doc: TdomDocument);
  public
    { Public-Deklarationen }
  end;

var
  Mainpage: TMainpage;

implementation

{$R *.DFM}

procedure TMainpage.UpdateTreeView(const doc: TdomDocument);
  procedure HandleNodeList(Parent:TTreeNode; DomNodeList:TDomNodeList);
  var
    i:integer;
    DomNode:TDomNode;
    tn:TTreeNode;
    astr:string;
  begin
    for i:=0 to pred(DomNodeList.length) do
    begin
      DomNode:=DomNodeList.Item(i);
      astr:=DomNode.NodeName;
      if DomNode.NodeValue <> '' then astr:=astr+' ['+DomNode.NodeValue+']';
      astr:= astr + ' ('+GetEnumName(TypeInfo(TdomNodeType), integer(DomNode.NodeType))+') ';
      if domNode.nodeType = ntText_Node
       then if TdomText(domNode).isWhitespaceInElementContent
         then astr:= astr + '-- Whitespace in element content';
      tn:=parent.owner.AddChildObject(Parent,astr,domNode);
      if assigned(DomNode.ChildNodes) then HandleNodeList(tn, DomNode.ChildNodes);
    end;
  end;
var
  root:TTreeNode;
begin
  // Quick and dirty: always build the tree completely.
  // Delphi 5 seems to have problems with this approach,
  // but I could not figure out why.
  treeview1.Items.BeginUpdate;
  try
    treeview1.Items.Clear;
    Root:= treeview1.Items.AddObject(nil,concat(doc.NodeName,' (',GetEnumName(TypeInfo(TdomNodeType), integer(Doc.NodeType)),') ',Doc.classname),doc);
    HandleNodeList(Root,doc.ChildNodes);
  finally
    treeview1.Items.EndUpdate;
  end;
end;

procedure TMainpage.OpenFile(Sender: TObject);
var
  UpTime: {$ifdef VER100} integer; {$else} cardinal; {$endif} // use cardinal in D4 and D5
  Doc: TdomDocument;
  Index: integer;
  DurationStr: string;
begin
  OpenDialog1.InitialDir := ExtractFileDir(Label1.caption);
  if OpenDialog1.Execute then begin
    Update;

    if not FileExists (OpenDialog1.FileName) then begin
      Label3.caption:= '';
      Label6.caption:= '';
      Memo2.text:= 'File not found!';
      Exit;
    end;

    Memo1.Clear;
    Memo1.Update;
    Memo2.Clear;
    Memo2.Update;
    Memo2.Lines.BeginUpdate;
    try
      with XmlToDomParser1 do begin
        BufferSize := StrToInt(RadioGroup2.Items[RadioGroup2.ItemIndex]);
        UpTime := GetTickCount;
        try
          Doc:= FileToDom(OpenDialog1.FileName);
          DurationStr := Format('%d ms', [GetTickCount - UpTime]);
          Index := TabSet1.Tabs.Add(ExtractFileName(Doc.SystemId));
          TabSet1.TabIndex:= index;
        except
          DurationStr := Format('%d ms', [GetTickCount - UpTime]);
          Memo2.Lines.Append('Document parsing abandoned.');
        end;
        Label3.caption := DurationStr;
      end; {with ...}
    finally
      Memo2.Lines.EndUpdate;
    end;
    if Memo2.text = '' then Memo2.text:= 'Document successfully parsed.';

  end; {if OpenDialog1.Execute ...}
end;

procedure TMainpage.TabSet1Click(Sender: TObject);
var
  doc: TdomDocument;
  durationStr: string;
  S: string;
  UpTime: {$ifdef VER100} integer; {$else} cardinal; {$endif} // use cardinal in D4 and D5
begin
  with Memo1 do begin
    clear;
    Update;
  end;
  Label1.caption:= '';
  Label3.caption:= '';
  Label6.caption:= '';
  treeview1.Items.Clear;
  if TabSet1.TabIndex > -1 then begin
    SpeedButton2.Enabled:= true;
    doc:= (XmlToDomParser1.DOMImpl.documents.Item(TabSet1.TabIndex) as TdomDocument);
    Label1.caption:= doc.systemId;

    Update;
    DomToXmlParser1.BufferSize:= StrToInt(RadioGroup3.Items[RadioGroup3.ItemIndex]);
    UpTime:= GetTickCount;
    DomToXmlParser1.writeToString(doc,'Latin1',S);
    durationStr := Format('%d ms', [GetTickCount - UpTime]);
    Label6.caption:= durationStr;
    with Memo1 do begin
      Text:= S;
      Update;
    end;

    UpdateTreeView(doc);
    Update;
  end;
end;

procedure TMainpage.CloseFile(Sender: TObject);
var
  doc: TdomDocument;
begin
  if TabSet1.TabIndex > -1 then begin
    with TabSet1 do begin
      doc:= (XmlToDomParser1.DOMImpl.documents.Item(TabIndex) as TdomDocument);
      with XmlToDomParser1.DOMImpl do begin
        freeDocument(doc);
        freeUnusedASModels;
      end;
      Tabs.Delete(TabIndex);
      if Tabs.Count = 0 then SpeedButton2.Enabled:= false;
    end;
  end;
end;

procedure TMainpage.TabControl1Change(Sender: TObject);
begin
  case TabControl1.TabIndex of
    0: begin Memo1.show; TreeView1.hide; Panel1.hide; end;
    1: begin Memo1.hide; TreeView1.show; Panel1.hide; end;
    2: begin Memo1.hide; TreeView1.hide; Panel1.show; end;
  end;
end;

procedure TMainpage.Button1Click(Sender: TObject);
var
  doc: TdomDocument;
  erOpt: TdomEntityResolveOption;
  invalidASModel: TdomASModel;
  ok: boolean;
begin
  doc:= (XmlToDomParser1.DOMImpl.documents.Item(TabSet1.TabIndex) as TdomDocument);
  if assigned(doc) then begin
    if RadioGroup1.ItemIndex = 0
      then erOpt:= erReplace
      else erOpt:= erExpand;
    memo2.Clear;
    if doc.ASModels.length = 0
      then ok:= doc.prepareASModels
      else ok:= true;
    if ok then begin
      if doc.validate(erOpt)
        then MessageDlg('No violations of validity constraints detected.',mtInformation,[mbOK],0)
        else MessageDlg('Document is invalid!',mtInformation,[mbOK],0);
    end else begin
      MessageDlg('DTD is invalid!',mtInformation,[mbOK],0);
      while doc.ASModels.length > 0 do begin
        invalidASModel:= doc.ASModels[pred(doc.ASModels.length)];
        XmlToDomParser1.DOMImpl.freeASModel(invalidASModel);
      end;
    end;
  end else MessageDlg('No active document!',mtError,[mbOK],0);
  TabSet1Click(Sender);
end;

procedure TMainpage.DomImplementation1ExternalParsedEntity(sender: TObject;
  const parentSystemId: WideString; var publicId, systemId: WideString;
  var stream: TStream; var action: TXmlParserAction);
var
  path: TFileName;
begin
  if systemId <> '' then begin
    try

      // Resolve path:
         path := concat(ExtractFileDir(parentSystemId),'\',SystemID);
      // This works only with relative system identifiers of
      // certain kinds.

      systemId:= path;

      stream:= TFileStream.Create(path, fmOpenRead);

      action:= paOK;
    except
      action:= paFail;
    end;
  end;
end;

procedure TMainpage.FormCreate(Sender: TObject);
begin
  Caption:= 'XDOM '+DomImplementation1.xdomVersion+' -- Example 03';
end;

procedure TMainpage.ReportError(sender: TObject; error: TdomError;
  var continue: Boolean);
var
  nodeNameReport: wideString;
begin
  nodeNameReport:= '';
  with error do begin
    with location do begin
      if assigned (relatedCMNode) then begin
        nodeNameReport:= concat(' -- CM-nodeName: ',relatedCMNode.nodeName);
      end;
      if assigned (relatedNode) then begin
        nodeNameReport:= concat(nodeNameReport,' -- nodeName: ',relatedNode.nodeName);
      end;
    end;
    language:= iso639_en;
    memo2.Lines.Add(concat(message,nodeNameReport));
  end;
end;

end.
