unit main;

// Code Example of using XDOM 2.4.0
// Delphi implementation
//
// You need XDOM 2.4.0 or above to use this source code.
// The latest version of XDOM can be found at "http://www.philo.de/xml/".
//
// This example  source code  shows  how to
// combine  a TXmlStandardDocReader  and  a
// TXmlStreamBuilder component to transcode
// an  XML file  to an equivalent  XML file
// using a different encoding scheme.

interface

uses
  XDOM_2_4, UnicodeConv, TypInfo,
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, Tabs, FileCtrl, ComCtrls, ExtCtrls, Grids;

type
  TMainpage = class(TForm)
    Label2: TLabel;
    Label1: TLabel;
    OpenDialog1: TOpenDialog;
    SpeedButton1: TSpeedButton;
    DomImplementation1: TDomImplementation;
    Edit1: TEdit;
    Label3: TLabel;
    ComboBox1: TComboBox;
    Label4: TLabel;
    XmlStandardDocReader1: TXmlStandardDocReader;
    XmlStreamBuilder1: TXmlStreamBuilder;
    SaveButton: TButton;
    SaveDialog1: TSaveDialog;
    procedure OpenFile(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure SaveFile(Sender: TObject);
  private
    { Private-Deklarationen }
  public
    { Public-Deklarationen }
  end;

var
  Mainpage: TMainpage;

implementation

{$R *.DFM}

procedure TMainpage.OpenFile(Sender: TObject);
begin
  if OpenDialog1.Execute
    then Edit1.Text:= OpenDialog1.FileName;
end;

procedure TMainpage.FormCreate(Sender: TObject);
begin
  Caption:= 'XDOM '+DomImplementation1.xdomVersion+' -- Example 09 -- Transcoder';
  ComboBox1.ItemIndex:= 0;  // Necessary, because Delphi 3 does not pulish the TCustomComboBox.ItemIndex property.
end;

procedure TMainpage.SaveFile(Sender: TObject);
var
  ParsingTime, UpTime: {$ifdef VER100} integer; {$else} cardinal; {$endif} // use cardinal in D4, D5, and D6
  OutputStream: TFileStream;
  InputStream: TFileStream;
  InputSrc: TXmlInputSource;
  Ok: boolean;
  OutputSrc: TXmlOutputSource;
begin
  if SaveDialog1.Execute then begin
    Update;

    if SaveDialog1.Filename = Edit1.Text then begin
      MessageDlg('Source file and target file must not be the same!',mtError, [mbOK], 0);
      exit;
    end;

    if not FileExists(Edit1.Text) then begin
      MessageDlg('Source file not found!',mtError, [mbOK], 0);
      exit;
    end;

    UpTime:= GetTickCount;

    // Transcode the XML File
    // ----------------------
    //
    // To write the output file to disk we use a TFileStream instance.  This
    // Stream is encapsulated in a TXmlOutputSource instance, which is a kind
    // of a Stream Adapter for the TXmlStreamBuilder.
    //
    OutputStream:= TFileStream.create(SaveDialog1.Filename,fmCreate); // First, we create the output FileStream.
    try
      OutputSrc:= TXmlOutputSource.create(OutputStream,4096); // Then, we create the OutputSource.
      try
        OutputSrc.newLine:= nltCRLF;                          // We want CRLF to encode line breaks.
        XmlStreamBuilder1.outputSource:= OutputSrc;           // Here, we attache the OutputSource to the XMLStreamBuilder.
        XmlStreamBuilder1.defaultEncoding:= ComboBox1.text;   // We specify the output encoding.
        InputStream:= TFileStream.create(Edit1.Text,fmOpenRead); // Now, we create a Stream for the Input Source.
        try
          InputSrc:= TXmlInputSource.create(InputStream,'',Edit1.Text,1,4096); // Here, we create the Input Source.
          try
            Ok:= XmlStandardDocReader1.parse(InputSrc); // Voila! We transcode the Input Source.
            OutputSrc.flushBuffer;  // Empty the output buffer, so everything byte is really sent to the output stream.
          finally
            InputSrc.free;
          end;
        finally
          InputStream.free;
        end;
      finally
        // This is important:  We need to set the outputSource property of the
        // TXmlStreamBuilder instance to nil, because, the TXmlStreamBuilder
        // instance is not being notified, when the TXmlOutputSource instance
        // is freed.
        XmlStreamBuilder1.outputSource:= nil;
        OutputSrc.free;
      end;
    finally
      OutputStream.free;
    end;

    ParsingTime:= GetTickCount-UpTime;
    Label2.caption:= Format('%d ms', [ParsingTime]);

    if not Ok then begin  // error while parsing?
      MessageDlg('An error occured while transcoding the file!',mtError, [mbOK], 0);
    end;

  end; {if OpenDialog1.Execute ...}
end;

end.
